"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DateString = require("./dateString");
const Leaf = require("leaf-ts");
class Queue extends Leaf.States {
    constructor(verbose = false) {
        super();
        this.verbose = verbose;
        this.tasks = [];
        this.data = {
            shouldPause: false
        };
        this.setState("waiting");
    }
    add(task) {
        for (let item of this.tasks) {
            if (item.equal(task)) {
                if (this.verbose) {
                    console.log(`Merging task ${task.toString()}`);
                }
                return false;
            }
        }
        this.tasks.push(task);
        this.feed("startSignal");
    }
    pause() {
        this.data.shouldPause = true;
    }
    atWaiting() {
        this.data.currentTask = null;
        this.consumeWhenAvailableMergeToLast("startSignal", () => {
            this.setState("working");
        });
    }
    atWorking(stale) {
        if (this.data.shouldPause) {
            this.setState("waiting");
            return;
        }
        let task = this.tasks.shift();
        if (!task) {
            this.events.emit("empty");
            this.data.currentTask = null;
            this.setState("waiting");
            return;
        }
        this.data.currentTask = task;
        let start = Date.now();
        task.exec((err) => {
            if (err) {
                console.error(err);
                this.setState("working");
                return;
            }
            if (stale()) {
                return;
            }
            let end = Date.now();
            console.log(DateString.genReadableDateString(), task.toString(), `done (${end - start + "ms"})`);
            this.setState("working");
            return;
        });
    }
}
exports.Queue = Queue;
//# sourceMappingURL=queue.js.map