import * as  DateString from "./dateString"
import * as Leaf from "leaf-ts"
import { Task } from "./task"
export class Queue extends Leaf.States {
    constructor(public readonly verbose = false) {
        super()
        this.setState("waiting")
    }
    tasks: Task[] = []
    add(task: Task) {
        for (let item of this.tasks) {
            if (item.equal(task)) {
                if (this.verbose) {
                    console.log(`Merging task ${task.toString()}`)
                }
                return false
            }
        }
        this.tasks.push(task)
        this.feed("startSignal")
    }
    data: {
        shouldPause?: boolean
        currentTask?: Task

    } = {
            shouldPause: false
        }
    pause() {
        this.data.shouldPause = true
    }
    atWaiting() {
        this.data.currentTask = null
        this.consumeWhenAvailableMergeToLast("startSignal", () => {
            this.setState("working")
        })
    }
    events: Leaf.EventEmitter<{
        state: string
        empty
    }>
    atWorking(stale) {
        if (this.data.shouldPause) {
            this.setState("waiting")
            return
        }
        let task = this.tasks.shift()
        if (!task) {
            this.events.emit("empty")
            this.data.currentTask = null
            this.setState("waiting")
            return
        }
        this.data.currentTask = task
        let start = Date.now()
        task.exec((err) => {
            if (err) {
                console.error(err)
                this.setState("working")
                return
            }
            if (stale()) {
                return
            }
            let end = Date.now()
            console.log(DateString.genReadableDateString(), task.toString(), `done (${end - start + "ms"})`)
            this.setState("working")
            return
        })
    }
}
