"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
const child_process = require("child_process");
const globalConfig = require("./globalConfig");
const pathModule = require("path");
class Task {
    constructor(path, rawCommand, option = {}) {
        this.path = path;
        this.rawCommand = rawCommand;
        this.option = option;
        this.extname = pathModule.extname(this.path);
        this.filename = pathModule.basename(this.path, this.extname);
        this.basename = this.filename + this.extname;
        this.directory = pathModule.dirname(this.path) + "/";
        this.command = this.rawCommand.replace(/\{filename\}/g, this.filename)
            .replace(/\{fullpath\}/g, this.path)
            .replace(/\{directory\}/g, this.directory)
            .replace(/\{basename\}/g, this.basename)
            .replace(/\{extname\}/g, this.extname);
        this.isDone = false;
        this.isExecuting = false;
    }
    equal(task) {
        return this.command = task.command;
    }
    exec(callback) {
        if (this.isDone) {
            callback(new errors_1.Errors.AlreadyDone());
            return;
        }
        if (this.isExecuting) {
            callback(new errors_1.Errors.AlreadyExecuting());
            return;
        }
        let option = {};
        if (globalConfig.shell) {
            option.shell = globalConfig.shell;
        }
        this.isExecuting = true;
        let cp = child_process.exec(this.command, option);
        cp.stdout.pipe(process.stdout);
        cp.stderr.pipe(process.stderr);
        cp.once("error", (err) => {
            callback(new errors_1.Errors.TaskFailed(`task ${this.toString()} failed with error ${err}`, { via: err }));
        });
        cp.once("exit", (code) => {
            this.isDone = true;
            this.isExecuting = false;
            if (code !== 0) {
                callback(new errors_1.Errors.TaskFailed(`task ${this.toString()} failed with code ${code}`));
                return;
            }
            callback();
        });
    }
    toString() {
        return `[Task:${pathModule.basename(this.path)} -> ${this.command.slice(0, 20) + "..."}]`;
    }
}
exports.Task = Task;
//# sourceMappingURL=task.js.map