import { Errors } from "./errors"
import * as child_process from "child_process"
import * as globalConfig from "./globalConfig"
import * as pathModule from "path"
type Callback<T = any, E = Error> = (error?: E, data?: T) => void
export class Task {
    extname = pathModule.extname(this.path)
    filename = pathModule.basename(this.path, this.extname)
    basename = this.filename + this.extname
    directory = pathModule.dirname(this.path) + "/"
    command = this.rawCommand.replace(/\{filename\}/g, this.filename)
        .replace(/\{fullpath\}/g, this.path)
        .replace(/\{directory\}/g, this.directory)
        .replace(/\{basename\}/g, this.basename)
        .replace(/\{extname\}/g, this.extname)
    constructor(public readonly path, public readonly rawCommand: string, public option: any = {}) {
    }
    equal(task: Task) {
        return this.command = task.command
    }
    isDone = false
    isExecuting = false
    exec(callback: Callback<null>) {
        if (this.isDone) {
            callback(new Errors.AlreadyDone())
            return
        }
        if (this.isExecuting) {
            callback(new Errors.AlreadyExecuting())
            return
        }
        let option: any = {}
        if (globalConfig.shell) {
            option.shell = globalConfig.shell
        }
        this.isExecuting = true
        let cp = child_process.exec(this.command, option)
        cp.stdout.pipe(process.stdout)
        cp.stderr.pipe(process.stderr)
        cp.once("error", (err) => {
            callback(new Errors.TaskFailed(`task ${this.toString()} failed with error ${err}`, { via: err }))
        })
        cp.once("exit", (code) => {
            this.isDone = true
            this.isExecuting = false
            if (code !== 0) {
                callback(new Errors.TaskFailed(`task ${this.toString()} failed with code ${code}`))
                return
            }
            callback()
        })
    }
    toString() {
        return `[Task:${pathModule.basename(this.path)} -> ${this.command.slice(0, 20) + "..."}]`
    }
}
